/*
 * CSE3001.  Compiler project based on "Modern Compiler Implementation
 * in Java" by Andrew W. Appel.
 *
 */

package canon;

import temp.Label;
import temp.Temp;
import tree.Stm;
import tree.Exp;
import tree.StmList;
import tree.ExpList;

class MoveCall extends Stm {
   final tree.TEMP dst;
   final tree.CALL src;
   MoveCall(tree.TEMP d, tree.CALL s) {dst=d; src=s;}
   public ExpList kids() {return src.kids();}
   public Stm build(ExpList kids) {
      return new tree.MOVE(dst, src.build(kids));
  }
}   
  
class ExpCall extends Stm {
   final tree.CALL call;
   ExpCall(tree.CALL c) {call=c;}
   public ExpList kids() {return call.kids();}
   public Stm build(ExpList kids) {
      return new tree.EXP(call.build(kids));
   }
}   


class StmExpList {
   final Stm stm;
   final ExpList exps;
   StmExpList(Stm s, ExpList e) {stm=s; exps=e;}
}

public class Canon {
  
   static boolean isNop(Stm a) {
      return a instanceof tree.EXP
	 && ((tree.EXP)a).exp instanceof tree.CONST;
   }

   static Stm seq(Stm a, Stm b) {
      if (isNop(a)) return b;
      else if (isNop(b)) return a;
      else return new tree.SEQ(a,b);
   }

   static boolean commute(Stm a, Exp b) {
      return isNop(a)
	 || b instanceof tree.NAME
	 || b instanceof tree.CONST;
   }

   static Stm do_stm(tree.SEQ s) { 
      return seq(do_stm(s.left), do_stm(s.right));
   }

   static Stm do_stm(tree.MOVE s) { 
      if (s.dst instanceof tree.TEMP 
	  && s.src instanceof tree.CALL) 
	 return reorder_stm(new MoveCall((tree.TEMP)s.dst,
					 (tree.CALL)s.src));
      else if (s.dst instanceof tree.ESEQ)
	 return do_stm(new tree.SEQ(((tree.ESEQ)s.dst).stm,
				    new tree.MOVE(((tree.ESEQ)s.dst).exp,
						  s.src)));
      else return reorder_stm(s);
   }

   static Stm do_stm(tree.EXP s) { 
      if (s.exp instanceof tree.CALL)
	 return reorder_stm(new ExpCall((tree.CALL)s.exp));
      else return reorder_stm(s);
   }

   static tree.Stm do_stm(tree.Stm s) {
      if (s instanceof tree.SEQ) return do_stm((tree.SEQ)s);
      else if (s instanceof tree.MOVE) return do_stm((tree.MOVE)s);
      else if (s instanceof tree.EXP) return do_stm((tree.EXP)s);
      else return reorder_stm(s);
   }

   static tree.Stm reorder_stm(tree.Stm s) {
      StmExpList x = reorder(s.kids());
      return seq(x.stm, s.build(x.exps));
   }

   static tree.ESEQ do_exp(tree.ESEQ e) {
      tree.Stm stms = do_stm(e.stm);
      tree.ESEQ b = do_exp(e.exp);
      return new tree.ESEQ(seq(stms,b.stm), b.exp);
   }

   static tree.ESEQ do_exp (Exp e) {
      if (e instanceof tree.ESEQ) return do_exp((tree.ESEQ)e);
      else return reorder_exp(e);
   }
         
   static tree.ESEQ reorder_exp (Exp e) {
      StmExpList x = reorder(e.kids());
      return new tree.ESEQ(x.stm, e.build(x.exps));
   }

   static StmExpList nopNull = new StmExpList(new tree.EXP(new tree.CONST(0)),null);

   static StmExpList reorder (ExpList exps) {
      if (exps==null) {
	 return nopNull;
      } else {
	 final Exp a = exps.head;
	 if (a instanceof tree.CALL) {
	    final Temp t = new Temp();
	    final Exp e = new tree.ESEQ(new tree.MOVE(new tree.TEMP(t), a),
				       new tree.TEMP(t));
	    return reorder(new ExpList(e, exps.tail));
	 } else {
	    tree.ESEQ aa = do_exp(a);
	    final StmExpList bb = reorder(exps.tail);
	    if (commute(bb.stm, aa.exp))
	       return new StmExpList(seq(aa.stm,bb.stm), 
				     new ExpList(aa.exp,bb.exps));
	    else {
	       final Temp t = new Temp();
	       return new StmExpList(
				     seq(aa.stm, 
					 seq(new tree.MOVE(new tree.TEMP(t),aa.exp),
					     bb.stm)),
				     new ExpList(new tree.TEMP(t), bb.exps));
	    }
	 }
      }
   }
        
   static StmList linear(tree.SEQ s, StmList l) {
      return linear(s.left,linear(s.right,l));
   }
   static StmList linear(tree.Stm s, StmList l) {
      if (s instanceof tree.SEQ) return linear((tree.SEQ)s, l);
      else return new StmList(s,l);
   }

   static public StmList linearize(Stm s) {
      return linear(do_stm(s), null);
   }
}
 
