/*
 * Florida Tech, CSE3001: Compiler Design.  Part of the compiler project
 * from "Modern Compiler Implementation in Java" by Andrew W. Appel.
 */

/*
   This class consolidates into a singile static function "transform" the three
   steps in canonicalizing a tree.Stm into a flat list of tree.Stm.
*/

package canon;

import error.ErrorMsg;
import parser.TigerParser;     // parser produced by CUP

import syntax.Exp;
import syntax.Print;

import semantics.Semantics;

import java_cup.runtime.Scanner;

import java.io.InputStream;
import java.io.FileInputStream;
import java.io.IOException;

import tree.Stm;
import tree.StmList;

import canon.Canon;
import canon.BasicBlocks;
import canon.TraceSchedule;

import translate.Fragment;
import translate.ProcFrag;

import java.util.LinkedList;
import java.util.Iterator;

public class Main {

   public static StmList transform(Stm s) {
      final StmList stms = Canon.linearize (s);
      final BasicBlocks bb = new BasicBlocks(stms);
      final StmList traces = (new TraceSchedule(bb)).stms;
      return traces;
   }

   /* Appel, Exercise 8.1, page 193.  */

   public static tree.TEMP t = new tree.TEMP (new temp.Temp ("t"));
   public static tree.Stm s = new tree.LABEL (new temp.Label("s"));
   public static tree.Exp e = new tree.CONST (2718);
   public static tree.Exp e1= new tree.CONST (1828);

   public static Stm exercise_81a =
      new tree.MOVE (t, new tree.ESEQ (s, e));

   public static Stm exercise_81b =
      new tree.MOVE (new tree.MEM (new tree.ESEQ (s, e1)),e);

    public static boolean verbose = System.getProperty ("verbose")!=null;

    public static void main (String args[]) throws IOException {
        final String filename = args[0];
        final ErrorMsg errorMsg = new ErrorMsg (filename);
        final InputStream is = new FileInputStream (filename);
        final TigerParser parser =  new TigerParser (is, errorMsg);
        try {
            final Exp e = (Exp) parser.parse().value;
            
            if(errorMsg.getErrors()>0) {
             System.out.println(errorMsg.getErrors() + " errors.");
             return;
            }
         
            is.close();
            
            final Semantics s = new Semantics(errorMsg);
            final LinkedList frags = s.transProg (e);


            for(Iterator i=frags.iterator(); i.hasNext(); ) {
                final Fragment f = (Fragment)i.next();
                if(f instanceof ProcFrag)  {
                    ProcFrag frag = (ProcFrag)f;

                    final tree.Print p = new tree.Print(System.out);
                    // add incoming/outgoing argument pseudo-code

                    final StmList stms = canon.Main.transform(frag.body);
                    
                    for (StmList sl = stms; sl!=null; sl=sl.tail)
                        p.prStm (sl.head);
                    
                }
            }
        }
        catch(Exception e) {
            
        }
        
        System.out.println(errorMsg.getErrors() + " errors.");
    }
}
