/*
 * CSE3001.  Compiler project based on "Modern Compiler Implementation
 * in Java" by Andrew W. Appel.
 *
 */

package canon;

import temp.Label;
import tree.Stm;
import tree.StmList;

import java.util.HashMap;

public class TraceSchedule {

   public final StmList stms;    // the result

   private final BasicBlocks theBlocks;
   private final HashMap table = new HashMap();

   private StmList getLast (StmList block) {
      StmList l=block;
      while (l.tail.tail!=null)  l=l.tail;
      return l;
   }

   private void trace (StmList l) {
      for(;;) {
	 final tree.LABEL lab = (tree.LABEL) l.head;
	 table.remove(lab.label);
	 final StmList last = getLast(l);
	 final Stm s = last.tail.head;
	 if (s instanceof tree.JUMP) {
	    tree.JUMP j = (tree.JUMP)s;
	    final StmList target = (StmList)table.get(j.targets.head);
	    if (j.targets.tail==null && target!=null) {
               last.tail=target;
	       l=target;
	    } else {
	       last.tail.tail=getNext();
	       return;
	    }
	 } else if (s instanceof tree.CJUMP) {
	    tree.CJUMP j = (tree.CJUMP)s;
	    final StmList t = (tree.StmList)table.get(j.iftrue);
	    final StmList f = (tree.StmList)table.get(j.iffalse);
	    if (f!=null) {
	       last.tail.tail=f; 
	       l=f;
	    } else if (t!=null) {
	       final tree.CJUMP cj =
		  new tree.CJUMP (tree.CJUMP.notRel(j.relop), j.left,j.right, j.iffalse, j.iftrue);
	       /*
	       last.tail.head=cj;
	       last.tail.tail=t;
	       */
	       last.tail = new StmList (cj, t);
	       l=t;
	    } else {
	       final Label ff = new Label();
	       final tree.CJUMP cj = new tree.CJUMP(j.relop,j.left,j.right, j.iftrue,ff);
	       final StmList x = new StmList(new tree.LABEL(ff),					       new StmList(new tree.JUMP(j.iffalse),
								getNext()));
	       /*
	       last.tail.head = cj;
	       last.tail.tail = x;
	       */
	       last.tail = new StmList (cj, x);
	       return;
	    }
	 } else {
	    throw new Error("Bad basic block in TraceSchedule");
	 }
      }
   }

   private StmList getNext() {
      if (theBlocks.blocks==null) {
	 return new StmList(new tree.LABEL(theBlocks.done), null);
      } else {
	 final StmList s = theBlocks.blocks.head;
	 tree.LABEL lab = (tree.LABEL)s.head;
	 if (table.get(lab.label) != null) {
	    trace(s);
	    return s;
         } else {
	    theBlocks.blocks = theBlocks.blocks.tail;
	    return getNext();
         }
      }
   }

   public TraceSchedule (BasicBlocks b) {
      theBlocks=b;
      for (StmListList l = b.blocks; l!=null; l=l.tail) {
	 table.put(((tree.LABEL)l.head.head).label, l.head);
      }
      stms=getNext();
   }        
}


