/*
 * CSE3001.  Compiler project based on "Modern Compiler Implementation
 * in Java" by Andrew W. Appel.
 *
 */

package parser;

import error.ErrorMsg;
import parser.TigerScanner;    // scanner produced by JLex
import parser.TigerParser;     // parser produced by CUP

import java_cup.runtime.Scanner;

import java.io.InputStream;
import java.io.FileInputStream;
import java.io.IOException;

/**
 ** A driver for the parser produced by CUP.
 **/

public class ParserMain {

   public static boolean verbose = System.getProperty ("verbose")!=null;

   public static void main (String args[]) throws IOException {
      final String filename = args[0];
      final ErrorMsg errorMsg = new ErrorMsg (filename);
      final InputStream is = new FileInputStream (filename);
      final Scanner lexer = new TigerScanner (is, errorMsg);
      final TigerParser p =  new TigerParser (lexer, errorMsg);
      try {
	 if (verbose) {
	    p.debug_parse();
	 } else {
	    p.parse();
	 }
      } catch (Exception e) {
	 e.printStackTrace (System.err);
      }
      System.out.println (filename + ", errors=" + errorMsg.getErrors());
   }
}
