/*
 * CSE3001.  Compiler project based on "Modern Compiler Implementation
 * in Java" by Andrew W. Appel.
 *
 */

package parser;

import error.ErrorMsg;

/*
 *  Though we have no parser yet, the interface must be correctly followed
 *  in anticipation of the CUP parser to come.  Thus we use the token type
 *  "Symbol" that CUP will and implement an interface "Scanner" that CUP
 *  expects.
 */
import java_cup.runtime.Scanner;
import java_cup.runtime.Symbol;

import parser.TigerScanner;    // scanner produced by JLex

import java.io.InputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Map;
import java.util.TreeMap;
import java.lang.reflect.Field;


/**
 ** A driver for the scanner produeced by JLex.
 ** Reflection is used give a verbose output with token names.
 **/

public class ScannerMain {

   public static boolean verbose = System.getProperty ("verbose")!=null;

   private final static Map tokens = new TreeMap ();
   static {
      try {
	 final Class c = Class.forName ("parser.TokenConstants");
	 final Field [] f = c.getDeclaredFields();
	 for (int i=0; i<f.length; i++) {
	    tokens.put (new Integer (f[i].getInt(null)), f[i].getName());
	 }
      } catch (Exception e) {
	 e.printStackTrace (System.err);
      }
   }

   public static void main (String args[]) throws Exception {
      final String filename = args[0];
      final ErrorMsg errorMsg = new ErrorMsg (filename);
      final InputStream is = new FileInputStream (filename);
      //      final Scanner lexer = new TigerScanner (is, errorMsg);
      final Scanner lexer = new TigerScanner (is, errorMsg);

      Symbol tok;
      do { 
	 tok=lexer.next_token();
	 if (verbose) {
	    System.out.print (tokens.get(new Integer(tok.sym)));
	    if (tok.value != null) System.out.print( " (" + tok.value + ")" );
	    System.out.println ();
	 }
      } while (tok.sym != TokenConstants.EOF);

      is.close();

      System.out.println (filename + ", errors=" + errorMsg.getErrors());

   }

}
