package regalloc;

import temp.Temp;
import temp.TempMap;

import tree.*;

import java.util.HashMap;
import java.util.List;

import frame.Frame;

// assigns each root a temp, which can be retrieved somehow

public class RegAlloc
{
    boolean debug=false;
    
    Cost  cost;
    Frame frame;
    public HashMap map; // root ==> temp
    int  reg;

    List registers;
    Queue used;
    Queue avail;

    private Temp pop()  { return (Temp)used.push(avail.pop()); }
    private void push() {
        if(!used.empty())
            avail.push(used.pop());
    }

    private void put(Object o) 
    {
        map.put(o, avail.peek());
    }
    
    public Temp get(Object o) 
    { return (Temp)map.get(o); }
    
    private void put(Object o, int i)
    {
        if(debug)
            System.out.println("REG: " + reg);
        put(o);
        //map.put(o, registers.get(i));
    }
    
    private int need(Object o) 
    { return cost.get(o); }
    
    public RegAlloc(StmList stms, Frame f) 
    {
        cost = new Cost(stms);  // build register cost of each root

        frame = f;
        registers = frame.usableRegs();

        used = new Queue();
        avail = new Queue();
        
        for(int i=registers.size()-1; i>=0; i--)
            avail.push(registers.get(i));
        
        map = new HashMap();
        reg = 0;
        
        for(StmList sl = stms; sl!=null; sl=sl.tail) {

            if(debug) {
                System.out.println("**********");
                (new tree.Print(System.out)).prStm(sl.head);
            }
            allocate(sl.head);
        }

        if(debug) {
            System.out.println("REGMAP  : " + map.size());
            System.out.println("COSTMAP : " + cost.hm.size());
        }
    }

    private void allocate(Object s) 
    {
        if(s==null)                 return;
        else if(s instanceof CJUMP) allocate((CJUMP)s);
        else if(s instanceof EXP)   allocate((EXP)s);
        else if(s instanceof JUMP)  allocate((JUMP)s);
        else if(s instanceof LABEL) allocate((LABEL)s);
        else if(s instanceof MOVE)  allocate((MOVE)s);
        else if(s instanceof NOP)   allocate((NOP)s);
        else if(s instanceof SEQ)   allocate((SEQ)s);
        else if(s instanceof BINOP) allocate((BINOP)s);
        else if(s instanceof CALL)  allocate((CALL)s);
        else if(s instanceof CONST) allocate((CONST)s);
        else if(s instanceof TEMP)  allocate((TEMP)s); 
        else if(s instanceof MEM)   allocate((MEM)s);
        else if(s instanceof NAME)  allocate((NAME)s); 
        else if(s instanceof ESEQ)  allocate((ESEQ)s);
        else if(s instanceof ExpList) allocate((ExpList)s);
        else throw new Error("Unknown class: " + s.getClass());
    }

    private void allocate(NOP n)   { solve(n);                  }
    private void allocate(EXP e)   { solve(e, e.exp);           }
    private void allocate(JUMP j)  { solve(j);                  }
    private void allocate(LABEL l) { solve(l);                  }
    private void allocate(CJUMP s) { solve(s, s.left, s.right); }
    private void allocate(MOVE m)  { solve(m, m.dst, m.src);    }
    private void allocate(SEQ s)   { solve(s, s.left, s.right); }
    private void allocate(CONST c) { solve(c);                  }
    private void allocate(TEMP t)  { solve(t);                  }
    private void allocate(NAME n)  { solve(n);                  }
    private void allocate(BINOP b) { solve(b, b.left, b.right); }
    private void allocate(MEM m)   { solve(m, m.exp);           }
    private void allocate(ESEQ e)  { solve(e, e.stm, e.exp);    }
    private void allocate(CALL c)  { solve(c, c.func, c.args);  }

    private void allocate(ExpList e) 
    {
        int oldreg = reg;
        for(ExpList list=e; list!=null; list=list.tail)
            allocate(list.head);
        //        reg = oldreg;
    }
    
    private void solve(Object p) 
    {
        if(debug)
            System.out.println("[SINGLE] " + p.getClass() + " : " + need(p));

        //        if(need(p)>0)
        //   reg++;//=need(p);
        //reg+=need(p);

        if(need(p)!=0)
            pop();
        put(p, reg);
    }
    
    private void solve(Object p, Object a) 
    {
        if(debug)
            System.out.println(p.getClass() + " : " + need(p));
        
        allocate(a);
        put(p, reg);
    }
    
    private void solve(Object p, Object a, Object b) 
    {
        if(debug)
            System.out.println(p.getClass() + " : " + need(p));
        
        if(need(a)>=need(b)) {
            allocate(a);
            allocate(b);
        }
        else {
            allocate(b);
            allocate(a);
        }

        //        if( (need(a) + need(b))>0)
        //   reg--;
        
        //        if(reg==0)
        //put(p, reg);
        //else

        //        int df = need(p)-max(need(a), need(b));
        //                    reg--;
                    //push();
        push();
        put(p, reg);
    }

    private final int max(int a, int b, int c)   { return max(a, max(b, c)); }
    private final int max(int a, int b)          { return (a>b ? a : b);     }
}
