package sparc;

import error.ErrorMsg;
import parser.TigerParser;     // parser produced by CUP

import syntax.Exp;

import semantics.Semantics;

import java.io.InputStream;
import java.io.FileInputStream;
import java.io.IOException;

import java.util.LinkedList;
import java.util.List;
import java.util.Iterator;

import translate.Fragment;
import translate.DataFrag;
import translate.ProcFrag;

import regalloc.RegAlloc;

import tree.StmList;

public class Main {

   public static boolean verbose = System.getProperty ("verbose")!=null;

   public static void main (String args[]) throws IOException {
      final String filename = args[0];
      final ErrorMsg errorMsg = new ErrorMsg(filename);
      final InputStream is = new FileInputStream(filename);
      final TigerParser parser =  new TigerParser(is, errorMsg);
      try {
	 final Exp e = (Exp) parser.parse().value;
	 is.close();

         if(errorMsg.getErrors()>0) {
           System.out.println(errorMsg.getErrors() + " errors.");
           return;
         }
         
         final Semantics s = new Semantics(errorMsg);
         final LinkedList frags = s.transProg(e);

         if(errorMsg.getErrors()>0) {
             System.out.println(errorMsg.getErrors() + " errors.");
             return;
         }
         
         for(Iterator i=frags.iterator(); i.hasNext(); ) {
             final Fragment f = (Fragment)i.next();

             if(f instanceof DataFrag) {
                 System.out.println(f);
             }
             else if(f instanceof ProcFrag)  {
             
                 ProcFrag frag = (ProcFrag)f;
                 
                 final StmList stms =
                     canon.Main.transform(frag.body); // linearize
         
                 RegAlloc ra = new RegAlloc(stms, ((ProcFrag)f).frame);

                 Codegen cg = new Codegen();
                 final List instructions = cg.codegen(stms, ra);
             
                 for(Iterator ii=instructions.iterator(); ii.hasNext(); )
                     System.out.println(ii.next());
             }
         }
      } catch(Exception e) {
        e.printStackTrace (System.err);
      }
      //      System.out.println(errorMsg.getErrors() + " errors.");
   }
}
