/*
 * Florida Tech, CSE3001.  Compiler project based on "Modern Compiler
 * Implementation in Java" by Andrew W. Appel.
 */

package symbol;

import symbol.Record;

/*
   Syntactically types are names (strings) and type expressions; cf syntax.TypeX
   and subclasses NameType, ArrayType, RecordType.

   This abstract class represents data types of the Tiger language inside the
   compiler.  Concrete subclasss Name, Array, and Record correspond to kinds of
   user-defined data types.  Predefined types INT, STRING, VOID, NIL, and ERROR
   are provided as constants.

   Important capabilities are:
      A. When are types equal or compatible?
      B. How to build mutually recursive types
*/
public abstract class Type {
   public Type actual()             { return this; }
   public boolean coerceTo (Type t) { return this==t.actual(); }

   /* The one and only Int subclass of Type */
   private static class Int extends Type {
      public String toString() { return "int"; }
   }
   public static final Type INT = new Type.Int ();

   /* The one and only StringType subclass of Type */
   private static class StringType extends Type {
      public String toString() { return "string"; }
   }
   public static final Type STRING = new Type.StringType ();

   /* Void indicates no value; a statement or subprocedure  */
   private static class Void extends Type {
      public String toString() { return "void"; }
   }
   public static final Type VOID = new Type.Void ();

   
   private static class Nil extends Type {
      public boolean coerceTo (final Type t) {
	 final Type a = t.actual();
	 return (a instanceof Record) || (a instanceof Type.Nil);
      }
      public String toString() { return "nil"; }
   }
   public static final Type NIL = new Type.Nil ();

   public static final Type ERROR = new Type.ERROR ();
   private static class ERROR extends Type {
      public boolean coerceTo (final Type t) {
	 /*
	   Assuming an error has been reported it seems best to consider
	   this type to be whatever the programmer needs it to be.
	 */
	 return (true);
      }
      public String toString() { return "ERROR"; }
   }
  
}
