/*
 * CSE3001.  Compiler project based on "Modern Compiler Implementation
 * in Java" by Andrew W. Appel.
 *
 */

package tree;

import tree.Stm;
import tree.ExpList;

import temp.TempMap;
import temp.DefaultMap;

import java.io.PrintStream;

public class Print extends Printer {

   final TempMap tmap;

   public Print (PrintStream o) {this (o, new DefaultMap());}
   public Print (PrintStream o, TempMap t) {super (o); tmap=t;}


   void prStm (SEQ s, int in) {
      indent (in); sayln("SEQ(");
      prStm (s.left,in+2); sayln(",");
      prStm (s.right,in+2); say(")");
   }

   void prStm (LABEL s, int in) {
      indent (in); say("LABEL "); say(s.label.toString());
   }

   void prStm(JUMP s, int in) {
      indent (in); sayln("JUMP("); prExp(s.exp, in+2); say(")");
   }

   void prStm(CJUMP s, int in) {
      indent (in); say("CJUMP("); 
      switch(s.relop) {
      case CJUMP.EQ: say("EQ"); break;
      case CJUMP.NE: say("NE"); break;
      case CJUMP.LT: say("LT"); break;
      case CJUMP.GT: say("GT"); break;
      case CJUMP.LE: say("LE"); break;
      case CJUMP.GE: say("GE"); break;
      case CJUMP.ULT: say("ULT"); break;
      case CJUMP.ULE: say("ULE"); break;
      case CJUMP.UGT: say("UGT"); break;
      case CJUMP.UGE: say("UGE"); break;
      default:	 throw new RuntimeException ("unknown op="+s.relop);
      }
      sayln (","); prExp (s.left, in+2); sayln (",");
      prExp (s.right, in+2); sayln (",");
      indent (in+2); say(s.iftrue.toString()); say(","); 
      say (s.iffalse.toString()); say(")");
   }

   void prStm(MOVE s, int in) {
      indent (in); sayln("MOVE("); prExp(s.dst,in+2); sayln(","); 
      prExp(s.src,in+2); say(")");
   }

   void prStm (EXP s, int in) {
      indent (in); sayln ("EXP("); prExp (s.exp, in+2); say (")"); 
   }

   void prStm (Stm s, int in) {
      if (s instanceof SEQ)        prStm((SEQ)s, in);
      else if (s instanceof LABEL) prStm((LABEL)s, in);
      else if (s instanceof JUMP)  prStm((JUMP)s, in);
      else if (s instanceof CJUMP) prStm((CJUMP)s, in);
      else if (s instanceof MOVE)  prStm((MOVE)s, in);
      else if (s instanceof EXP)   prStm((EXP)s, in);
      else if (s instanceof NOP)   prStm((NOP)s, in);
      else throw new RuntimeException (s.getClass().getName());
   }

   void prExp(BINOP e, int in) {
      indent (in); say("BINOP("); 
      switch(e.binop) {
      case BINOP.PLUS: say("PLUS"); break;
      case BINOP.MINUS: say("MINUS"); break;
      case BINOP.MUL: say("MUL"); break;
      case BINOP.DIV: say("DIV"); break;
      case BINOP.AND: say("AND"); break;
      case BINOP.OR: say("OR"); break;
      case BINOP.LSHIFT: say("LSHIFT"); break;
      case BINOP.RSHIFT: say("RSHIFT"); break;
      case BINOP.ARSHIFT: say("ARSHIFT"); break;
      case BINOP.XOR: say("XOR"); break;
      default:
         throw new Error("Print.prExp.BINOP");
      }
      sayln(",");
      prExp(e.left,in+2); sayln(","); prExp(e.right,in+2); say(")");
   }

   void prExp (MEM e, int in) {
      indent (in);
      sayln("MEM("); prExp(e.exp,in+2); say(")");
   }

   void prExp (TEMP e, int in) {
      indent (in); say("TEMP "); 
      say(tmap.tempMap(e.temp));
   }

   void prExp (ESEQ e, int in) {
      indent (in); sayln("ESEQ(");
      prStm(e.stm,in+2); sayln(",");
      prExp(e.exp,in+2); say(")");

   }

   void prExp (NAME e, int in) {
      indent (in); say("NAME "); say(e.label.toString());
   }

   void prExp (CONST e, int in) {
      indent (in); say("CONST "); say(String.valueOf(e.value));
   }

   void prExp(CALL e, int in) {
      indent (in); sayln("CALL(");
      prExp (e.func, in+2);
      for (ExpList a = e.args; a!=null; a=a.tail) {
	 sayln (",");
	 prExp (a.head, in+2); 
      }
      say(")");
   }

  void prStm(NOP e, int in) {
    indent(in); sayln("NOP");
  }
  
   void prExp (Exp e, int in) {
      if (e instanceof BINOP)      prExp((BINOP)e, in);
      else if (e instanceof MEM)   prExp((MEM)e, in);
      else if (e instanceof TEMP)  prExp((TEMP)e, in);
      else if (e instanceof ESEQ)  prExp((ESEQ)e, in);
      else if (e instanceof NAME)  prExp((NAME)e, in);
      else if (e instanceof CONST) prExp((CONST)e, in);
      else if (e instanceof CALL)  prExp((CALL)e, in);
      else throw new RuntimeException (e.getClass().getName());
   }

   public void prStm(Stm s) {prStm(s,0); say("\n");}
   public void prExp(Exp e) {prExp(e,0); say("\n");}

}
